#include <proto.h>

#include "gtscrollbar.h"
#include "gtgroup.h"
#include "gtbutton.h"
#include "gtsymbol.h"
#include "gtscroller.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtScrollBar class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

#define SCT_ORIENT	0
#define SCT_START		1
#define SCT_END			2
#define SCT_PAGE		3
#define SCT_VALUE		4
#define SCT_REDUCE	5
#define SCT_REPDRAG	6
#define SCT_MAX_NR	7

/************************************************************************\
*
* Create and init scrollbar.
*
\************************************************************************/
GtScrollBar *GtMkScrollBar(char *n,GtManager *p,GtTag *tgs)
{
	GtScrollBar *obj;
	
	if(!(obj=new GtScrollBar()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtScrollBar object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init scrollbar.
*
\************************************************************************/
GtScrollBar *GtMkScrollBar(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtScrollBar *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkScrollBar(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty scrollbar.
*
\************************************************************************/
GtScrollBar::GtScrollBar(void)
{
	grp=NULL;
	scr=NULL;
	up=NULL;
	down=NULL;
	orient=GTO_HORIZ;
	step=1;
}

/************************************************************************\
*
* Destroy scrollbar.
*
\************************************************************************/
void GtScrollBar::Destroy(void)
{
	if(grp)
		GtDestroy(grp);
	GtWidget::Destroy();
}

/************************************************************************\
*
* Setup scrollbar.
*
\************************************************************************/
sbool GtScrollBar::Init(char *nm,GtManager *p,GtTag *tgs)
{
	GtTag *t,ts[SCT_MAX_NR+1];
	ulong sz=10;
	long idel=250,del=100;
	ubyte ap=GTAP_TOGETHER;
	int n;
	
	if(!GtWidget::Init(nm,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_ORIENT,GTRT_ORIENT,&orient);
	app->GetRes(this,GTR_SBSZ,GTRT_LONG,&sz);
	app->GetRes(this,GTR_APLACE,GTRT_APLACE,&ap);
	for(n = 0; n < SCT_MAX_NR; n++)
		ts[n].tag = GT_IGNORE;
	ts[SCT_MAX_NR].tag=GT_DONE;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Orient:
				orient=t->d.val;
				if(orient>GTO_LAST_NR)
					orient=GTO_HORIZ;
				break;
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_ArrowPlace:
				ap=t->d.val;
				if(ap>GTAP_LAST_NR)
					ap=GTAP_TOGETHER;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
			case GT_Step:
				step=t->d.val;
				break;
			case GT_Start:
				ts[SCT_START].tag = GT_Start;
				ts[SCT_START].d.val = t->d.val;
				break;
			case GT_End:
				ts[SCT_END].tag = GT_End;
				ts[SCT_END].d.val = t->d.val;
				break;
			case GT_Page:
				ts[SCT_PAGE].tag = GT_Page;
				ts[SCT_PAGE].d.val = t->d.val;
				break;
			case GT_Value:
				ts[SCT_VALUE].tag = GT_Value;
				ts[SCT_VALUE].d.val = t->d.val;
				break;
			case GT_ReportDrag:
				ts[SCT_REPDRAG].tag = GT_ReportDrag;
				ts[SCT_REPDRAG].d.val = t->d.val;
				break;			
			case GT_ReduceInterval:
				ts[SCT_REDUCE].tag = GT_ReduceInterval;
				ts[SCT_REDUCE].d.val = t->d.val;
				break;
		}
	}
	if(!(grp=GtMkGroup("scrgroup",p,
			GT_Orient,orient,
			(orient==GTO_HORIZ ? GT_MinYSize : GT_MinXSize),sz,
			(orient==GTO_HORIZ ? GT_MaxYSize : GT_MaxXSize),sz,
			GT_DONE)))
		return(FALSE);
	ts[SCT_ORIENT].tag = GT_Orient;
	ts[SCT_ORIENT].d.val = orient;
	if(!(scr=GtMkScroller("scr",grp,ts)))
		return(FALSE);
	if(!(up=GtMkButton("uparrow",grp,
			GT_Repeat,TRUE,
			GT_InitialDelay,idel,
			GT_Delay,del,
			GT_Weight,0,
			GT_MinXSize,sz,
			GT_MinYSize,sz,
			GT_DONE)))
		return(FALSE);
	up->AddCB(GT_Activate,ArrowUp,this);
	if(!GtMkSymbol("upsym",up,
			GT_Symbol,GTST_SARROW,
			GT_Direction,(orient==GTO_HORIZ ? GTD_LEFT : GTD_UP),
			GT_DONE))
		return(FALSE);
	if(!(down=GtMkButton("downarrow",grp,
			GT_Repeat,TRUE,
			GT_InitialDelay,idel,
			GT_Delay,del,
			GT_Weight,0,
			GT_MinXSize,sz,
			GT_MinYSize,sz,
			GT_DONE)))
		return(FALSE);
	down->AddCB(GT_Activate,ArrowDown,this);
	if(!GtMkSymbol("downsym",down,
			GT_Symbol,GTST_SARROW,
			GT_Direction,(orient==GTO_HORIZ ? GTD_RIGHT : GTD_DOWN),
			GT_DONE))
		return(FALSE);
	if(ap==GTAP_APART)
		grp->PositionChild(up,0);
	return(TRUE);
}

/************************************************************************\
*
* Set scrollbar attributes.
*
\************************************************************************/
sbool GtScrollBar::Set(GtTag *tgs)
{
	GtTag *t,ts[SCT_MAX_NR+1];
	ulong sz=0;
	long idel=0,del=0;
	sbool ch=FALSE;
	int n;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	for(n = 0; n < SCT_MAX_NR; n++)
		ts[n].tag = GT_IGNORE;
	ts[SCT_MAX_NR].tag=GT_DONE;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
			case GT_Step:
				step=t->d.val;
				break;
			case GT_Start:
				ts[SCT_START].tag = GT_Start;
				ts[SCT_START].d.val = t->d.val;
				ch = TRUE;
				break;
			case GT_End:
				ts[SCT_END].tag = GT_End;
				ts[SCT_END].d.val = t->d.val;
				ch = TRUE;
				break;
			case GT_Page:
				ts[SCT_PAGE].tag = GT_Page;
				ts[SCT_PAGE].d.val = t->d.val;
				ch = TRUE;
				break;
			case GT_Value:
				ts[SCT_VALUE].tag = GT_Value;
				ts[SCT_VALUE].d.val = t->d.val;
				ch = TRUE;
				break;
			case GT_ReportDrag:
				ts[SCT_REPDRAG].tag = GT_ReportDrag;
				ts[SCT_REPDRAG].d.val = t->d.val;
				break;			
			case GT_ReduceInterval:
				ts[SCT_REDUCE].tag = GT_ReduceInterval;
				ts[SCT_REDUCE].d.val = t->d.val;
				ch = TRUE;
				break;
		}
	}
	if(ch)
		scr->Set(ts);
	if(idel || del)
	{
		up->SetV(
			(idel ? GT_InitialDelay : GT_IGNORE),idel,
			(del ? GT_Delay : GT_IGNORE),del,
			GT_DONE);
		down->SetV(
			(idel ? GT_InitialDelay : GT_IGNORE),idel,
			(del ? GT_Delay : GT_IGNORE),del,
			GT_DONE);
	}
	if(sz)
	{
		grp->SetV(
			(orient==GTO_HORIZ ? GT_MinYSize : GT_MinXSize),sz,
			(orient==GTO_HORIZ ? GT_MaxYSize : GT_MaxXSize),sz,
			GT_DONE);
		up->SetV(GT_MinXSize,sz,GT_MinYSize,sz,GT_DONE);
		down->SetV(GT_MinXSize,sz,GT_MinYSize,sz,GT_DONE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Get scrollbar attributes.
*
\************************************************************************/
void GtScrollBar::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_ScrBarSize:
				*ptr=up->GetA(GT_MinXSize,0);
				break;
			case GT_InitialDelay:
				*ptr=up->GetA(GT_InitialDelay,0);
				break;
			case GT_Delay:
				*ptr=up->GetA(GT_Delay,0);
				break;
			case GT_Step:
				*ptr=step;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtScrollBar::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtScrollBar")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Add callback function.
*
\************************************************************************/
void GtScrollBar::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	if(tag==GT_ValueChange)
		scr->AddCB(tag,cb,ud);
	else
		GtWidget::AddCB(tag,cb,ud);
}

/************************************************************************\
*
* Remove callback function.
*
\************************************************************************/
void GtScrollBar::RemCB(ulong tag,GtCallB cb)
{
	if(tag==GT_ValueChange)
		scr->RemCB(tag,cb);
	else
		GtWidget::RemCB(tag,cb);
}

/************************************************************************\
*
* Up arrow button callback.
*
\************************************************************************/
int GtScrollBar::ArrowUp(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollBar *sb;
	ulong v,m;
	
	obj=obj;
	cd=cd;
	sb=(GtScrollBar *)ud;
	sb->scr->GetV(GT_Value,&v,GT_Start,&m,GT_DONE);
	if(v>m)
	{
		sb->scr->SetV(GT_Value,v-sb->step,GT_DONE);
		sb->scr->Notify(GT_Activate);
	}
	return(GTCB_CONT);
}

/************************************************************************\
*
* Down arrow button callback.
*
\************************************************************************/
int GtScrollBar::ArrowDown(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrollBar *sb;
	ulong v,m;
	
	obj=obj;
	cd=cd;
	sb=(GtScrollBar *)ud;
	sb->scr->GetV(GT_Value,&v,GT_End,&m,GT_DONE);
	if(v<m)
	{
		sb->scr->SetV(GT_Value,v+sb->step,GT_DONE);
		sb->scr->Notify(GT_Activate);
	}
	return(GTCB_CONT);
}
